# When set to $true, the script will import the certificate into the appropriate stores if needed
# When set to $false, the script simply validates that the certificate is present in the needed stores, and returns a boolean
$Remediate = $false

# See function help for instructions on getting the Thumbprint
$CodeSigningCertificateThumbprint = '----Certificate Thumbprint Here----'

# See function help for instructions on getting the the Encoded Cert String
$EncodedCertString = '----Encoded Cert String Here----'

function Register-CodeSigningCertificate {
    <#
        .SYNOPSIS
            Import a certificate into the Trusted Publishers and Trusted Root cert store
        .DESCRIPTION
            This function is used to validate or import a certificate into both the Trusted Publishers 
            and the Trusted Root certificate store. This is useful when you need to manage a Code Signing
            certificate. The function accepts a 'EncodedCertString' which will be a base64 encoded value
            which equates the the actual certificate file. This is used to allow for the script to be used
            without provided additional files. 

            The main use case is for use in a Configuration Item within Configuration Manager.
        .PARAMETER Remediate
            A boolean that determines if the certificate will be imported if not found.
        .PARAMETER CodeSigningCertificateThumbprint
            The thumbprint of the certificate which will be useed to find the certificate in the
            two certificate stores. The 'Thumbprint' of a certificate can be retrieved from the details
            tab of the properties of a certificate.
        .PARAMETER EncodedCertString
            A string that is a base64 represntation of the certificate file. This can be retrieved with the
            below code snippet where ExportedCert.cer is your Code Signing certificate file, and is located.
            in the directory where the command is being ran from.

            Set-Clipboard -Value ([System.Convert]::ToBase64String((Get-Content -Path .\ExportedCert.cer -Encoding Byte)))

            Note: You don't need to have the Private Key marked as exportable to do this.
        .EXAMPLE
            C:\PS>
            Example of how to use this cmdlet
        .EXAMPLE
            C:\PS>
            Another example of how to use this cmdlet
        .NOTES
            FileName:    Register-CodeSigningCertificate.ps1
            Author:      Cody Mathis
            Contact:     @CodyMathis123
            Created:     2020-05-11
            Updated:     2020-05-11
#>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory = $true)]
        [bool]$Remediate,
        [Parameter(Mandatory = $true)]
        [string]$CodeSigningCertificateThumbprint,
        [Parameter(Mandatory = $true)]
        [string]$EncodedCertString
    )
    # A hashtable of the targeted certificate stores, and the result of searching them
    $CertStoreSearchResult = @{
        TrustedPublisher = $false
        Root             = $false
    }

    foreach ($CertStoreName in @("TrustedPublisher", "Root")) {
        $CertStore = [System.Security.Cryptography.X509Certificates.X509Store]::new($CertStoreName, "LocalMachine")
        $CertStore.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadOnly)

        switch ($CertStore.Certificates.Thumbprint) {
            $CodeSigningCertificateThumbprint {
                $CertStoreSearchResult[$CertStoreName] = $true
            }
        }
        $CertStore.Close()
    }

    foreach ($Result in $CertStoreSearchResult.GetEnumerator()) {
        switch ($Result.Value) {
            $false {
                switch ($Remediate) {
                    $true {
                        $CertStore = [System.Security.Cryptography.X509Certificates.X509Store]::new($Result.Key, "LocalMachine")
                        $CertStore.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadWrite)
                
                        $CertificateByteArray = [System.Convert]::FromBase64String($EncodedCertString)
                        $Certificate = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2
                        $Certificate.Import($CertificateByteArray)
                    
                        $CertStore.Add($Certificate)
                        $CertStore.Close()
                    }
                    $false {
                        return $false
                    }
                }   
            }
        }
    }

    return $true
}

Register-CodeSigningCertificate -Remediate $Remediate -CodeSigningCertificateThumbprint $CodeSigningCertificateThumbprint -EncodedCertString $EncodedCertString